const { Warning, PlainText } = wp.editor;
const { select, dispatch, subscribe } = wp.data;
//import Button from '@wordpress/components'; TODO: refactor without global variable usage
const { Component, RawHTML } = wp.element;
const { __ } = wp.i18n;

import IntervalAutoSave from './IntervalAutoSave';
import EditorAutoSave from './EditorAutoSave';
import compareVersions from 'compare-versions';

const BLOCK_SELECTED_HTML_CLASS = 'setka-editor-selected';

export default class SetkaEditor extends Component {

    /**
     * @type {EditorAssets}
     */
    assets = null;

    /**
     * @type {AdminMenu}
     */
    adminMenu = null;

    /**
     * @type {IntervalAutoSave}
     */
    _autoSave;

    /**
     * @type {boolean}
     */
    _editorRunning = false;

    state = {
        renderComponent: 'editor', // 'html-editor' | 'warning' | 'assets-error'
    };

    /**
     * @type {null|boolean}
     */
    isEditorSidebarOpened = null;

    _storageChangesUnsubscribe = null;

    /**
     * Store previous content for Setka Editor (used in componentDidUpdate).
     * @type {Object}
     */
    contentSavedIntoProps;

    constructor(props) {
        super(props);
        this.initialize = this.initialize.bind(this);
        this.saveContent = this.saveContent.bind(this);
        this.onChange = this.onChange.bind(this);
        this.assets = props.attributes.assets;
        this.adminMenu = props.attributes.adminMenu;

        if (this._isEditorSupportsAutoSave()) {
            this._autoSave = new EditorAutoSave();
        } else {
            this._autoSave = new IntervalAutoSave(this.saveContent);
        }
    }

    componentDidMount() {
        if (document.readyState === 'complete') {
            this.tryInitialize();
        } else {
            window.addEventListener('DOMContentLoaded', () => {this.tryInitialize()});
        }
    }

    tryInitialize() {
        this.assets.assetsPromise.then(
            () => {this.initialize()},
            exception => {this.showAssetsLoadError()}
        )
    }

    componentWillUnmount() {
        //window.removeEventListener('DOMContentLoaded', this.initialize);
        //wp.oldEditor.remove( `editor-${ this.props.id }` );

        this.disableEditorAndAutoSave();
    }

    initialize() {
        // Fold admin menu (left column) if page less than 1340px and greater than 782 in width
        // On medium screens all panels automatically collapse.
        if(document.body.clientWidth < 1340 && select('core/viewport').isViewportMatch('>= medium')) {
            this.adminMenu.fold();

            if(document.body.clientWidth < 1216) {
                dispatch('core/edit-post').closeGeneralSidebar();
            }
        }

        try {
            if('undefined' === typeof this.props.attributes.content || '' === this.props.attributes.content) {
                this.enableEditorAndAutoSave();
            } else {
                let assets = this.prepareAssetsFromAttributes();
                this.enableEditorAndAutoSave();
                this.replaceContentAndAssets(this.props.attributes.content, assets);
            }
            this.setupIsSelected();
        } catch (error) {
            this.showErrors(error);
        }

        return this;
    }

    onChange() {
        if (this.isEditorSidebarOpened !== select('core/edit-post').isEditorSidebarOpened()) {
            setTimeout(() => {
                window.SetkaEditor.trigger('editor:width');
            }, 0);
            this.isEditorSidebarOpened = select('core/edit-post').isEditorSidebarOpened();
        }
    }

    enableEditorAndAutoSave() {
        this.contentSavedIntoProps = this.props.attributes.content;
        window.SetkaEditor.start(this._prepareEditorConfig(), this.assets.assets);
        this._autoSave.enable();
        this._editorRunning = true;

        this.isEditorSidebarOpened = select('core/edit-post').isEditorSidebarOpened();
        this._storageChangesUnsubscribe = subscribe(this.onChange);

        return this;
    }

    disableEditorAndAutoSave() {
        if(this.is2Version()) {
            window.SetkaEditor.stop();
        }

        this._autoSave.disable();
        this._editorRunning = false;

        if('function' === typeof this._storageChangesUnsubscribe) {
            this._storageChangesUnsubscribe();
            this._storageChangesUnsubscribe = null;
        }

        return this;
    }

    /**
     * @return {Object}
     * @private
     */
    _prepareEditorConfig() {
        switch (this._autoSave.constructor.name) {
            case IntervalAutoSave.name:
                return this.assets.config;

            case EditorAutoSave.name:
                return {...this.assets.config, ...{onPostContentChange: this.saveContent}};

            default:
                throw new Error('Unknown auto save type: ' + this._autoSave.constructor.name);
        }
    }

    /**
     * Find Setka Editor assets from block attributes.
     *
     * @throws {AssetsNotFoundError} If required theme or layout not found in assets storage.
     *
     * @return {{
     *     theme: {string},
     *     layout: {string}
     * }}
     */
    prepareAssetsFromAttributes() {
        return this.assets.findAssetsForIds(
            this.props.attributes.setkaEditorTheme,
            this.props.attributes.setkaEditorLayout
        );
    }

    /**
     * Find Setka Editor assets from content string.
     *
     * @param content {string} HTML markup.
     *
     * @throws {AssetsNotFoundError} If required theme or layout not found in assets storage.
     *
     * @return {Object} Assets
     */
    prepareAssetsFromContent(content) {
        return this.assets.findAssetsForContent(content);
    }

    /**
     * This method created for support undo/redo events.
     *
     * @param prevProps {Object} Previous properties value.
     *
     * @see handleAssets
     */
    componentDidUpdate(prevProps) {
        this.setupIsSelected();

        if(!this.isEditorRunning()) {
            return;
        }

        // Because we save content every 5 seconds (not on every update) there is an opportunity
        // to start replacing content in the editor even if the editor itself contains a newer version of content.
        if('string' === typeof this.props.attributes.content
            &&
            this.contentSavedIntoProps !== this.props.attributes.content
        ) {
            try {
                this.replaceContentAndAssets(
                    this.props.attributes.content,
                    this.prepareAssetsFromContent(this.props.attributes.content)
                );
            } catch (error) {
                this.showErrors(error);
            }
        }
    }

    /**
     * Saves content from editor into Gutenberg (WordPress).
     *
     * @return {this} For chain calls.
     */
    saveContent() {
        let theme = window.SetkaEditor.getCurrentTheme();
        let attributes = {
            content: window.SetkaEditor.getHTML(),
            setkaEditorTheme: theme.id,
            setkaEditorLayout: window.SetkaEditor.getCurrentLayout().id,
        };

        this.contentSavedIntoProps = attributes.content;
        this.props.setAttributes(attributes);

        return this;
    }

    /**
     * Replace content and assets in Setka Editor.
     *
     * @return {this} For chain calls.
     */
    replaceContentAndAssets(content, assets) {

        if('object' !== typeof assets) {
            return this;
        }

        if('string' !== typeof assets.theme || 'string' !== typeof assets.layout) {
            return this;
        }

        if(this.is2Version()) {
            window.SetkaEditor.setTheme(assets.theme);
            window.SetkaEditor.setLayout(assets.layout);
        } else {
            //window.SetkaEditor.post.setTheme(assets.theme);
            window.__store.setTheme(assets.theme);

            //window.SetkaEditor.post.setLayout(assets.layout);
            window.__store.setLayout(assets.layout);
        }

        window.SetkaEditor.replaceHTML(content);

        return this;
    }

    render() {
        switch (this.state.renderComponent) {
            case 'warning':
            default:
                return <Warning key="setka-warning" actions={[
                    <wp.components.Button key="show-html-editor" isLarge onClick={ this.showHtmlEditor }>{ __('Show HTML', 'setka-editor') }</wp.components.Button>,
                    <wp.components.Button key="support" isLarge onClick={ () => { window.open('https://editor-help.setka.io') } }>{ __('Help Center', 'setka-editor') }</wp.components.Button>,
                    <wp.components.Button key="settings" isLarge onClick={ () => { window.open(setkaEditorGutenbergModules.settingsUrl) } }>{ __('Plugin Settings', 'setka-editor') }</wp.components.Button>,
                ]}>
                    <RawHTML>{ this.assets.getAssetsError().message }</RawHTML>
                </Warning>;

            case 'assets-error':
                return <Warning key="setka-assets-error">
                    <RawHTML>{ this.assets.getAssetsError().message }</RawHTML>
                </Warning>;

            case 'html-editor':
                return <PlainText
                    value={ this.props.attributes.content }
                    onChange={ (content) => this.props.setAttributes({ content }) } />;

            case 'editor':

                return <div>
                    <div key="editor" id="setka-editor" className="stk-editor"/>
                </div>;
        }
    }

    showHtmlEditor = () => {
        this.setState({renderComponent: 'html-editor'});
    }

    showErrors(error) {
        this.disableEditorAndAutoSave();
        this.setState({renderComponent: 'warning'});

        if(error.theme) {
            dispatch('core/notices').createErrorNotice(error.theme.message, {__unstableHTML: true});
        }
        if(error.layout) {
            dispatch('core/notices').createErrorNotice(error.layout.message, {__unstableHTML: true});
        }

        return this;
    }

    showAssetsLoadError() {
        this.setState({renderComponent: 'assets-error'});
    }

    /**
     * @return {boolean}
     */
    is2Version() {
        return 'string' === typeof window.SetkaEditor.version;
    }

    /**
     * @return {string}
     * @private
     */
    _getEditorVersion() {
        return window.SetkaEditor.version;
    }

    /**
     * @return {boolean}
     * @private
     */
    _isEditorSupportsAutoSave() {
        if (!this.is2Version()) {
            return false;
        }

        let matches = /^[0-9.]+/.exec(this._getEditorVersion());
        let version = matches ? matches[0] : null;

        if (!version) {
            return false;
        }

        try {
            return compareVersions.compare('2.5.2', version, '<=');
        } catch (exception) {
            return false;
        }
    }

    /**
     * @return {string}
     */
    getHtmlFromEditor() {
        return window.SetkaEditor.getHTML();
    }

    /**
     * @return {boolean}
     */
    isEditorRunning() {
        return this._editorRunning;
    }

    /**
     * Setup additional CSS class for <html> tag if block selected.
     *
     * @return {this} For chain calls.
     */
    setupIsSelected() {
        if(this.props.isSelected) {
            jQuery(document.documentElement).addClass(BLOCK_SELECTED_HTML_CLASS);
        } else {
            jQuery(document.documentElement).removeClass(BLOCK_SELECTED_HTML_CLASS);
        }
        return this;
    }
}
