<?php
namespace Setka\Editor\Admin\Pages\Settings;

use Korobochkin\WPKit\DataComponents\NodeInterface;
use Setka\Editor\Admin\Options\EditorAccessPostTypesOption;
use Setka\Editor\Admin\Options\SrcsetSizesOption;
use Setka\Editor\Admin\Options\WebhooksEndpointOption;
use Setka\Editor\Admin\Options\WhiteLabelOption;
use Setka\Editor\Admin\User\Capabilities\UseEditorCapability;

class SettingsBuilder
{
    /**
     * @var Settings
     */
    private $settings;

    /**
     * @var array
     */
    private $dataComponentsMap = array(
        WebhooksEndpointOption::class => 'setEndpoint',
        WhiteLabelOption::class       => 'setWhiteLabel',
    );

    /**
     * @var NodeInterface[]
     */
    private $dataComponents;

    /**
     * SettingsBuilder constructor.
     *
     * @param SrcsetSizesOption $srcsetSizesOption
     * @param EditorAccessPostTypesOption $editorAccessPostTypesOption
     * @param WebhooksEndpointOption $webhooksEndpoint
     * @param WhiteLabelOption $whiteLabelOption
     */
    public function __construct(
        SrcsetSizesOption $srcsetSizesOption,
        EditorAccessPostTypesOption $editorAccessPostTypesOption,
        WebhooksEndpointOption $webhooksEndpoint,
        WhiteLabelOption $whiteLabelOption
    ) {
        $this->dataComponents = array(
            SrcsetSizesOption::class           => $srcsetSizesOption,
            EditorAccessPostTypesOption::class => $editorAccessPostTypesOption,
            WebhooksEndpointOption::class      => $webhooksEndpoint,
            WhiteLabelOption::class            => $whiteLabelOption,
        );
    }

    /**
     * @return Settings
     */
    public function build()
    {
        $this->settings = new Settings();
        $this->buildPostTypes()->buildRoles()->buildSrcsetSizes()->buildDataComponents();
        return $this->settings;
    }

    /**
     * @return $this
     */
    private function buildPostTypes()
    {
        $postTypes = $this->dataComponents[EditorAccessPostTypesOption::class]->get();
        $value     = array();

        foreach ($postTypes as $postType) {
            $value[] = new PostType($postType);
        }

        $this->settings->setPostTypes($value);

        return $this;
    }

    /**
     * @return $this
     */
    private function buildRoles()
    {
        $roles = $this->getRoles();

        if (empty($roles)) {
            return $this;
        }

        $rolesSelected = array();

        foreach ($roles as $index => $role) {
            if (isset($role['capabilities'][UseEditorCapability::NAME])
                &&
                true === $role['capabilities'][UseEditorCapability::NAME]
            ) {
                $rolesSelected[] = $index;
            }
        }

        $this->settings->setRoles($rolesSelected);

        return $this;
    }

    /**
     * @return array[]
     */
    private function getRoles()
    {
        return get_editable_roles();
    }

    /**
     * @return $this
     */
    private function buildSrcsetSizes()
    {
        $sizes = $this->dataComponents[SrcsetSizesOption::class]->get();
        $value = array();

        foreach ($sizes as $size) {
            $value[] = new ImageSize($size);
        }

        $this->settings->setSrcsetSizes($value);

        return $this;
    }

    /**
     * @return $this
     */
    private function buildDataComponents()
    {
        foreach ($this->dataComponentsMap as $name => $setter) {
            call_user_func(array($this->settings, $setter), $this->dataComponents[$name]->get());
        }
        return $this;
    }
}
