<?php
namespace Setka\Editor\Admin\Pages\Settings;

use Setka\Editor\Admin\Options\WebhooksEndpointOption;
use Setka\Editor\Plugin;
use Setka\Editor\Service\Config\ImageSizesConfig;
use Setka\Editor\Service\Config\PluginConfig;
use Setka\Editor\Service\Constraints\WordPressNonceConstraint;
use Symfony\Component\Form\AbstractType;
use Symfony\Component\Form\FormBuilderInterface;
use Symfony\Component\Form\Extension\Core\Type;
use Symfony\Component\OptionsResolver\OptionsResolver;
use Symfony\Component\Validator\Constraints;

class SettingsType extends AbstractType
{
    /**
     * @param FormBuilderInterface $builder
     * @param array $options
     */
    public function buildForm(FormBuilderInterface $builder, array $options)
    {
        $this->buildPostTypes($builder)->buildRoles($builder);

        if (!$options['vip']) {
            $builder->add('srcset_sizes', Type\ChoiceType::class, array(
                'choices' => $this->getSrcsetSizesChoices($options['srcset_sizes']),
                'choice_label' => 'getId',
                'choice_value' => 'getId',
                'multiple' => true,
                'expanded' => true,
                'choice_attr' => 'buildAttributes',
            ));
        }

        $builder->add('endpoint', Type\ChoiceType::class, array(
            'label' => __('Webhooks Endpoint', Plugin::NAME),
            'choices' => array(
                'wp-admin/admin-post.php (default)' => WebhooksEndpointOption::TYPE_POST,
                'wp-admin/admin-ajax.php' => WebhooksEndpointOption::TYPE_AJAX,
            ),
            'multiple' => false,
            'expanded' => false,
        ));

        $builder->add('white_label', Type\CheckboxType::class, array(
            'label' => __('Credits', Plugin::NAME),
            'required' => false,
        ));

        $builder->add('nonce', Type\HiddenType::class, array(
            'data' => wp_create_nonce(Plugin::NAME . '-save-settings'),
            'constraints' => array(
                new Constraints\NotBlank(),
                new WordPressNonceConstraint(array('name' => Plugin::NAME . '-save-settings')),
            ),
        ));

        $builder->add('submit', Type\SubmitType::class, array(
            'label' => __('Save Changes', Plugin::NAME),
            'attr' => array('class' => 'button button-primary'),
        ));
    }

    /**
     * @param OptionsResolver $resolver
     */
    public function configureOptions(OptionsResolver $resolver)
    {
        $resolver->setRequired('srcset_sizes');
        $resolver->setAllowedTypes('srcset_sizes', 'array');

        $resolver->setDefault('vip', false);
        $resolver->setAllowedTypes('vip', 'bool');
    }

    /**
     * @param FormBuilderInterface $builder
     * @return $this
     */
    private function buildPostTypes(FormBuilderInterface $builder)
    {
        $builder->add(
            'post_types',
            Type\ChoiceType::class,
            array(
                'choices' => $this->getPostTypeChoices(),
                'choice_label' => 'getName',
                'choice_value' => 'getId',
                'multiple' => true,
                'expanded' => true,
            )
        );

        return $this;
    }

    private function getPostTypeChoices()
    {
        $postTypes = $this->getPostTypes();
        $choices   = array();


        foreach ($postTypes as $key => $value) {
            $postTypeObject = $this->getPostTypeObject($value);
            $choices[]      = new PostType($postTypeObject->name, $postTypeObject->labels->name);
        }

        return $choices;
    }

    /**
     * @return array
     */
    private function getPostTypes()
    {
        return PluginConfig::getAvailablePostTypes();
    }

    /**
     * @param $postType string
     * @return \WP_Post_Type
     */
    private function getPostTypeObject($postType)
    {
        return get_post_type_object($postType);
    }

    /**
     * @param FormBuilderInterface $builder
     * @return $this
     */
    private function buildRoles(FormBuilderInterface $builder)
    {
        $roles = $this->getRoles();

        if (empty($roles)) {
            return $this;
        }

        $choices = array();

        foreach ($roles as $key => $value) {
            $choices[$value['name']] = $key;
        }

        $builder->add(
            'roles',
            Type\ChoiceType::class,
            array(
                'choices' => $choices,
                'multiple' => true,
                'expanded' => true,
            )
        );

        return $this;
    }

    /**
     * @return array[]
     */
    private function getRoles()
    {
        return get_editable_roles();
    }

    /**
     * @param $sizes array
     * @return array
     */
    private function getSrcsetSizesChoices(array $sizes)
    {
        $values = array();

        foreach ($sizes as $name => $size) {
            $values[] = new ImageSize(
                $name,
                $size[ImageSizesConfig::SETTING_WIDTH],
                $size[ImageSizesConfig::SETTING_HEIGHT],
                $size[ImageSizesConfig::SETTING_CROP]
            );
        }

        return $values;
    }
}
