<?php
namespace Setka\Editor\CLI\Commands;

use Setka\Editor\Admin\Cron\AMPStylesCronEvent;
use Setka\Editor\Admin\Cron\AMPStylesQueueCronEvent;
use Setka\Editor\Service\AMP\AMPStatus;
use WP_CLI as Console;

class AMPCommand extends \WP_CLI_Command
{
    /**
     * @var AMPStylesCronEvent
     */
    protected $ampStylesCronEvent;

    /**
     * @var AMPStylesQueueCronEvent
     */
    protected $ampStylesQueueCronEvent;

    /**
     * @var AMPStatus
     */
    private $AMPStatus;

    /**
     * AMPSyncCommand constructor.
     * @param AMPStylesCronEvent $ampStylesCronEvent
     * @param AMPStylesQueueCronEvent $ampStylesQueueCronEvent
     * @param AMPStatus $ampStatus
     */
    public function __construct(
        AMPStylesCronEvent $ampStylesCronEvent,
        AMPStylesQueueCronEvent $ampStylesQueueCronEvent,
        AMPStatus $ampStatus
    ) {
        $this->ampStylesCronEvent      = $ampStylesCronEvent;
        $this->ampStylesQueueCronEvent = $ampStylesQueueCronEvent;
        $this->AMPStatus               = $ampStatus;
    }

    /**
     * Show AMP styles sync status.
     *
     * @alias st
     *
     * @when after_wp_load
     */
    public function status()
    {
        \WP_CLI\Utils\format_items('yaml', $this->getCurrentSettings(), array('Name', 'Value'));
    }

    /**
     * @return array
     */
    private function getCurrentSettings()
    {
        $options = $this->AMPStatus->getCurrentSettings();
        $data    = array();
        foreach ($options as $name => &$value) {
            $data[] = array(
                'Name' => $name,
                'Value' => $value,
            );
        }
        return $data;
    }

    /**
     * Restart AMP styles sync
     *
     * @alias res
     *
     * @when after_wp_load
     */
    public function restart()
    {
        $this->ampStylesCronEvent->restart();
        $this->ampStylesQueueCronEvent->restart();
        $this->status();
        Console::success('Restarted.');
    }

    /**
     * Disable AMP styles sync
     *
     * @alias dis
     *
     * @when after_wp_load
     */
    public function disable()
    {
        $this->ampStylesCronEvent->unscheduleAll()->getAmpStylesManager()->resetSync();
        $this->ampStylesQueueCronEvent->unscheduleAll();
        $this->status();
        Console::success('Disabled.');
    }

    /**
     * Enable AMP styles sync
     *
     * @alias en
     *
     * @when after_wp_load
     */
    public function enable()
    {
        $this->restart();
        $this->status();
        Console::success('Enabled.');
    }

    /**
     * Delete all AMP files and options.
     *
     * @when after_wp_load
     */
    public function delete()
    {
        try {
            $this->ampStylesCronEvent->getAmpStylesManager()->deleteAllFiles()->resetSync();
        } catch (\Exception $exception) {
            $message = 'Error while deleting posts:' . PHP_EOL .
                'Exception name: ' . get_class($exception) . PHP_EOL .
                'Exception code: ' . $exception->getCode() . PHP_EOL .
                'Exception message: ' . $exception->getMessage();
            Console::error($message);
        }

        $this->ampStylesCronEvent->unscheduleAll();
        $this->ampStylesQueueCronEvent->unscheduleAll();

        Console::success('All files was removed.');
    }

    /**
     * Show all AMP files.
     *
     * @when after_wp_load
     */
    public function show()
    {
        $items = $this->AMPStatus->getCSSList();

        if (empty($items)) {
            Console::log('AMP files not found.');
            return;
        }

        \WP_CLI\Utils\format_items('table', $items, array('ID', 'post_name', 'post_status', 'post_type'));
    }

    /**
     * Count different AMP files.
     *
     * @when after_wp_load
     */
    public function count()
    {
        $counters = $this->AMPStatus->getCSSCounters();
        foreach ($counters as $counterName => $counter) {
            if (is_a($counter, \Exception::class)) {
                $counter = $counter->getMessage();
            }
            Console::log($counterName . ': ' . $counter);
        }
    }
}
