<?php
namespace Setka\Editor\Service\AMP;

use Setka\Editor\Plugin;
use Setka\Editor\Service\AMP\Traits\XPathFactoryTrait;

class ImageSanitizer extends \AMP_Base_Sanitizer
{
    use XPathFactoryTrait;

    /**
     * @var \DOMElement Current image.
     */
    private $image;

    /**
     * @var integer Id of current image (attachment).
     */
    private $id;

    /**
     * @inheritdoc
     */
    public function sanitize()
    {
        /**
         * @var $posts \DOMNodeList
         * @var $post \DOMElement
         * @var $images \DOMNodeList
         */
        $posts = $this->setupXPath()->createSetkaPosts();

        foreach ($posts as $post) {
            $images = $this->createImages($post);

            foreach ($images as $this->image) {
                try {
                    $this->validateImage()
                         ->removeStkReset()
                         ->removeSizes()
                         ->detectCurrentImageId()
                         ->widthAndHeightAttributes()
                         ->srcSet();
                } catch (\Exception $exception) {
                    continue;
                }
            }
        }
    }

    /**
     * Detect current img id.
     *
     * ID stored in data-image-id attribute if post created in Setka Editor 2 version, otherwise in id.
     *
     * @throws \RuntimeException If id not found.
     *
     * @return $this
     */
    public function detectCurrentImageId()
    {
        if ($this->image->hasAttribute('data-image-id')) {
            $id = $this->image->getAttribute('data-image-id');
        } else {
            $idAttr = $this->image->getAttribute('id');
            $id     = filter_var($idAttr, FILTER_SANITIZE_NUMBER_INT); // attribute format id='image-1325'
        }

        if (!is_string($id) || empty($id)) {
            throw new \RuntimeException();
        }

        $id = absint($id);

        if ($id <= 0) {
            throw new \RuntimeException();
        }

        $this->id = $id;

        return $this;
    }

    /**
     * Remove stk-reset CSS class.
     *
     * @return $this
     */
    public function removeStkReset()
    {
        $classes = trim($this->image->getAttribute('class'));

        if (empty($classes)) {
            return $this;
        }

        $classes = explode(' ', $classes);

        if (empty($classes)) {
            return $this;
        }

        $index = array_search('stk-reset', $classes, true);

        if (false !== $index) {
            unset($classes[$index]);

            if (empty($classes)) {
                $this->image->removeAttribute('class');
            } else {
                $this->image->setAttribute('class', implode(' ', $classes));
            }
        }

        return $this;
    }

    /**
     * @return $this
     */
    private function removeSizes()
    {
        $this->image->removeAttribute('sizes');
        return $this;
    }

    /**
     * Add width and height attributes for img.
     *
     * @throws \RuntimeException
     *
     * @return $this
     */
    public function widthAndHeightAttributes()
    {
        if (!isset($this->id)) {
            throw new \RuntimeException();
        }

        if ($this->image->hasAttribute('width') || $this->image->hasAttribute('height')) {
            return $this;
        }

        $meta = wp_get_attachment_metadata($this->id);

        if (isset($meta['width']) && isset($meta['height'])) {
            $this->image->setAttribute('width', $meta['width']);
            $this->image->setAttribute('height', $meta['height']);
        }

        return $this;
    }

    /**
     * Add srcset attribute for img.
     *
     * @throws \RuntimeException
     *
     * @return $this
     */
    public function srcSet()
    {
        if (!isset($this->id)) {
            throw new \RuntimeException();
        }

        if ($this->image->hasAttribute('srcset')) {
            return $this;
        }

        $srcSet = wp_get_attachment_image_srcset($this->id, Plugin::NAME.'-1000');

        if (is_string($srcSet)) {
            $this->image->setAttribute('srcset', $srcSet);
        }

        return $this;
    }

    /**
     * @return \DOMNodeList
     */
    private function createSetkaPosts()
    {
        return $this->xpath->query('//div[contains(@class, "stk-post")]');
    }

    /**
     * @param \DOMElement $post
     *
     * @return \DOMNodeList
     */
    private function createImages(\DOMElement $post)
    {
        return $this->xpath->query('.//img', $post);
    }

    /**
     * @throws \DomainException
     * @return $this
     */
    private function validateImage()
    {
        if (!is_a($this->image, \DOMElement::class)) {
            throw new \DomainException();
        }

        $this->id = null;

        return $this;
    }
}
